#! /usr/bin/python -O
# -*- coding: utf-8 -*-
#
# A Translator Bot MUC client.
# Copyright (C) 2014 MiKael NAVARRO
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""A Translator Bot MUC client.
Copyright (C) 2014 MiKael NAVARRO

This XMPP client logs in, joins the room using the nick 'transbot' 
and responds to message addressed to it.  If another occupant writes 
'fr-es: bonjour', it will return 'es: hola'.

"""

# Include directives
from twisted.python import log
from twisted.words.protocols.jabber.jid import JID
from wokkel.client import XMPPClient
from wokkel.muc import MUCClient

from subprocess import check_output


#
# We will need to create the python class: MUCTranslateBot.
# All this does it import the python module we will be using and
# extend the muc.MUCCLient class.
#
# This example uses MUCClient instead of the protocol-only MUCClientProtocol 
# so that it can hook into its receivedGroupChat.  MUCClient implements 
# groupChatReceived and makes a distinction between messages setting the subject, 
# messages that a part of the room's conversation history, and 'live' messages. 
# In this case, we only want to inspect and respond to the 'live' messages.
#
class MUCTranslateBot(MUCClient):
    """I join a room and respond to translation requests."""

    def __init__(self, roomJID, nick):
        MUCClient.__init__(self)
        self.roomJID = roomJID
        self.nick = nick


    def connectionInitialized(self):
        """Once authorized, join the room.

        If the join action causes a new room to be created, the room will be
        locked until configured. Here we will just accept the default
        configuration by submitting an empty form using configure, which
        usually results in a public non-persistent room."""

        def joinedRoom(room):
            if room.locked:
                # Just accept the default configuration. 
                return self.configure(room.roomJID, {})

        MUCClient.connectionInitialized(self)

        d = self.join(self.roomJID, self.nick)
        d.addCallback(joinedRoom)
        d.addCallback(lambda _: log.msg("Joined room"))
        d.addErrback(log.err, "Join failed")


    def receivedGroupChat(self, room, user, message):
        """Called when a groupchat message was received.

        Check if the message was addressed to my nick. Respond by sending 
        translated message to the room addressed to the sender."""

        if message.body.startswith(u"fr-es:"):
            nick, text = message.body.split(':', 1)
            text = text.strip().lower()

            # Call apertium to translate messages
            output = check_output("echo '%s' | apertium fr-es" % text, shell=True)
            output = unicode(output, "utf-8")  # convert output to unicode

            self.groupChat(self.roomJID, u"es: %s" % output)

        elif message.body.startswith(u"es-fr:"):
            nick, text = message.body.split(':', 1)
            text = text.strip().lower()

            # Call apertium to translate messages
            output = check_output("echo '%s' | apertium es-fr" % text, shell=True)
            output = unicode(output, "utf-8")  # convert output to unicode

            self.groupChat(self.roomJID, u"fr: %s" % output)


# The bot is now complete, we will need a conf file to start it up.
# To do this I will use a .tac file for twisted => transbot.tac
